#!/usr/bin/python

caps = list(range(ord("A"), ord("Z")+1))
xchars = [ord(c) for c in "acemnorsuvwxz"]

def convert_def(fn):
	src = file(fn)

	result = ""
	header = ""
	cap_height = []
	x_height = []
	for line in src.readlines():
		line = line.strip()
		if not line or line[0]=='#':
			continue

		parts = line.split()

		if parts[0]=="font":
			tw, th, fh, fa, fd = map(int, parts[1:])
			header += "native_size %d;\n"%fh
			header += "ascent %.3f;\n"%(float(fa)/fh)
			header += "descent %.3f;\n"%(float(fd)/fh)
		elif parts[0]=="glyph":
			g, x, y, w, h, ox, oy, a = map(int, parts[1:])
			result += "glyph %d\n{\n"%g
			result += "\ttexcoords %f %f %f %f;\n"%(float(x)/tw, float(th-y-h)/th, float(x+w)/tw, float(th-y)/th)
			result += "\tsize %.3f %.3f;\n"%(float(w)/fh, float(h)/fh)
			result += "\toffset %.3f %.3f;\n"%(float(ox)/fh, float(oy)/fh)
			result += "\tadvance %.3f;\n"%(float(a)/fh)
			result += "};\n"

			if g in caps:
				cap_height.append(h)
			elif g in xchars:
				x_height.append(h)
		elif parts[0]=="kern":
			l, r, d = map(int, parts[1:])
			result += "kerning %d %d %.3f;\n"%(l, r, float(d)/fh)

	if cap_height:
		header += "cap_height %.3f;\n"%(float(cap_height[len(cap_height)*2/3])/fh)
	if x_height:
		header += "x_height %.3f;\n"%(float(x_height[len(x_height)*2/3])/fh)

	return header+result

def make_font(fn, size, ch_range, autohinter, margin, padding):
	import maketex
	import os

	cmd = "ttf2png \"%s\" -o makefont-tmp.png -d makefont-tmp.def -t -p -s %d -m %d -n %d"%(fn, size, margin, padding)
	if ch_range:
		cmd += " -r %d,%d"%ch_range
	if autohinter:
		cmd += " -a"
	if os.system(cmd):
		raise Exception("Could not execute ttf2png")

	result = "texture\n{\n"
	result += maketex.make_tex("makefont-tmp.png", wrap="CLAMP_TO_EDGE")
	result += "};\n"
	result += convert_def("makefont-tmp.def")

	os.unlink("makefont-tmp.png")
	os.unlink("makefont-tmp.def")

	return result

def parse_range(s):
	return tuple(map(int, s.split(',', 1)))

if __name__=="__main__":
	import sys
	import argparse

	parser = argparse.ArgumentParser()
	parser.add_argument("-s", "--size", default=10, type=int, metavar="NUM", help="Font size in pixels")
	parser.add_argument("-a", "--autohinter", action="store_const", const=True, default=False, help="Force autohinter")
	parser.add_argument("-m", "--margin", default=0, type=int, metavar="NUM", help="Margin around image edge in pixels")
	parser.add_argument("-n", "--padding", default=1, type=int, metavar="NUM", help="Padding between glyphs in pixels")
	parser.add_argument("-r", "--range", type=parse_range, metavar="START,END", help="Range of code points to include")
	parser.add_argument("ttf", metavar="font.ttf", help="TrueType file to read")
	parser.add_argument("outfile", help="MspGL font file to write")

	args = parser.parse_args()

	out = file(args.outfile, "w")
	out.write(make_font(args.ttf, args.size, args.range, args.autohinter, args.margin, args.padding))
