#include <msp/core/refptr.h>
#include <msp/datafile/collection.h>
#include <msp/strings/format.h>
#include "material.h"
#include "program.h"
#include "programdata.h"
#include "tag.h"
#include "technique.h"
#include "texture.h"

using namespace std;

namespace Msp {
namespace GL {

RenderPass &Technique::add_pass(const Tag &tag)
{
	return insert_unique(passes, tag, RenderPass())->second;
}

bool Technique::has_pass(const Tag &tag) const
{
	return passes.count(tag);
}

const RenderPass &Technique::get_pass(const Tag &tag) const
{
	return get_item(passes, tag);
}

void Technique::replace_texture(const string &slot, const Texture &tex)
{
	for(PassMap::iterator i=passes.begin(); i!=passes.end(); ++i)
	{
		int index = i->second.get_texture_index(slot);
		if(index<0)
			continue;
		i->second.set_texture(index, &tex);
	}
}

bool Technique::has_shaders() const
{
	for(PassMap::const_iterator i=passes.begin(); i!=passes.end(); ++i)
		if(i->second.get_shader_program())
			return true;
	return false;
}


Technique::Loader::Loader(Technique &t):
	DataFile::CollectionObjectLoader<Technique>(t, 0)
{
	init();
}

Technique::Loader::Loader(Technique &t, Collection &c):
	DataFile::CollectionObjectLoader<Technique>(t, &c)
{
	init();
}

void Technique::Loader::init()
{
	add("inherit", &Loader::inherit);
	add("pass", &Loader::pass);
}

void Technique::Loader::inherit(const string &n)
{
	obj.passes = get_collection().get<Technique>(n).get_passes();
	InheritLoader ldr(obj, get_collection());
	load_sub_with(ldr);
}

void Technique::Loader::pass(const string &n)
{
	RenderPass p;
	if(coll)
		load_sub(p, get_collection());
	else
		load_sub(p);

	insert_unique(obj.passes, n, p);
}


Technique::InheritLoader::InheritLoader(Technique &t, Collection &c):
	DataFile::CollectionObjectLoader<Technique>(t, &c)
{
	add("material", &InheritLoader::material);
	add("texture", &InheritLoader::texture);
}

void Technique::InheritLoader::material(const string &pass_tag, const string &name)
{
	RenderPass &pass = get_item(obj.passes, pass_tag);
	const Material &mat = get_collection().get<Material>(name);
	if(const Material *base_mat = pass.get_material())
	{
		for(PassMap::iterator i=obj.passes.begin(); i!=obj.passes.end(); ++i)
			if(i->second.get_material()==base_mat)
				i->second.set_material(&mat);
	}
	else
		pass.set_material(&mat);
}

void Technique::InheritLoader::texture(const string &slot, const string &name)
{
	obj.replace_texture(slot, get_collection().get<Texture>(name));
}

} // namespace GL
} // namespace Msp
